var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { CloudStorage, uuid, HTTP, DBCache, reservedKey, documentPath, checkValidObject } from '../internal';
const defaultConfig = {
    database: '',
    cacheExpiration: 500,
    indices: false,
    retries: 3
};
export class NearDB {
    /**
     * Constructor to setup config, and path, and required checks.
     * @param config configuration to initialize NearDB instance
     */
    constructor(config) {
        /** Check if config exists */
        /** Overwrites config param with default configuration */
        this.config = Object.assign(Object.assign({}, defaultConfig), config);
        // TODO: define the type of storage in the config
        this.adapter = new CloudStorage(this.config);
        // Creates instanceid
        this.instanceId = uuid();
    }
    /**
     * Static method to create a new instance of NearDB class.
     * @param config configuration to initiatlize NearDB instance
     * @returns an initialized instance of NearDB with the config
     */
    static database(config) {
        return new NearDB(Object.assign(defaultConfig, config));
    }
    /**
     * Returns a new instance of collection
     * @param key expects key for collection
     * @returns an instance of the collection class
     */
    collection(key) {
        return new Collection(this, key);
    }
}
// Constants used for document update
NearDB.field = {
    deleteValue: 'NEARDB.FIELD.DELETE'
};
export class Collection {
    constructor(instance, key, path) {
        // Check if this is a reserved keyword
        if (reservedKey(key)) {
            throw new Error(key + ': is a reserved keyword');
        }
        this.instance = instance;
        // Copy value of path before passing, to avoid polluting scope
        const newPath = path ? [...path] : [];
        newPath.push({
            type: 'collection',
            key
        });
        this.path = newPath;
        this.dbPath = documentPath(this.path);
    }
    doc(key) {
        return new Document(this.instance, key, this.path);
    }
    /**
     * Adds a document to a collection by generating an id for the doc
     * @param value expects payload to be stored for the document
     * @returns a promise for the payload of the saved doc
     */
    add(value) {
        return new Document(this.instance, uuid(), this.path).set(value);
    }
}
export class Document {
    constructor(instance, key, path) {
        // Check if this is a reserved keyword
        if (reservedKey(key)) {
            throw new Error(key + ': is a reserved keyword');
        }
        // Copy value of path before passing, to avoid poluting scope
        const newPath = [...path];
        // Push new pathItem into the path array
        newPath.push({
            type: 'document',
            key
        });
        this.path = newPath;
        this.dbPath = documentPath(this.path);
        this.instance = instance;
        // Sets default cache value
        this.cache = new DBCache(this.instance.config.cacheExpiration);
    }
    /**
     * Creates a new instance of Collection
     * @param key expects key for collection
     * @returns an instance of the collection class
     */
    collection(key) {
        return new Collection(this.instance, key, this.path);
    }
    /**
     * Sets document data from the path provided in the scope.
     * @param value expects payload to be stored for the document
     * @returns payload of the document requested
     */
    set(value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                checkValidObject(value);
                const payload = yield this.instance.adapter.set(value, this.dbPath);
                return payload;
            }
            catch (err) {
                throw err;
            }
        });
    }
    /**
     * Gets document data from the path provided in the scope.
     * @param options sets options on how to get documents
     * @returns payload of the document requested
     */
    get(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let data;
            const config = this.instance.config;
            const source = options && options.source ? options.source : null;
            const cdnUrl = config && config.cdn && config.cdn.url;
            // Conditional if there is a cache
            const isCache = source === null && this.cache.exists();
            // Conditional if its an edge
            const isEdge = cdnUrl && (source === 'edge' || source === null);
            // Conditional if its from origin
            // let isOrigin = source === 'origin'
            try {
                if (isCache)
                    return (data = this.cache.get());
                if (isEdge)
                    return (data = yield this.getFromEdge());
                return (data = yield this.getFromOrigin());
            }
            catch (err) {
                throw err;
            }
        });
    }
    /**
     * Update some fields of the document without overwriting entire document
     * @param value expects payload to be stored for the document
     * @returns a promise for the payload requested
     */
    update(value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let doc;
                // Get the latest document from the origin
                doc = yield this.get({ source: 'origin' });
                // Loop through all property for custom object actions
                for (const prop in value) {
                    if (value[prop] === NearDB.field.deleteValue) {
                        // If has deleteValue action, delete the prop
                        delete value[prop];
                        delete doc[prop];
                    }
                }
                // Updates document
                const payload = yield this.set(Object.assign(Object.assign({}, doc), value));
                // Stores payload in local cache
                this.cache.set(payload);
                return payload;
            }
            catch (err) {
                throw err;
            }
        });
    }
    /**
     * Removes document store in the path of instance
     * @returns empty object
     */
    delete() {
        return this.instance.adapter.remove(this.dbPath);
    }
    /**
     * Makes a get request to the CDN url
     * @param path path to attach to cdn url on the request
     * @returns json object from the request
     */
    getFromEdge() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const http = HTTP.create({
                    baseURL: this.instance.config.cdn.url,
                    timeout: 15000,
                    headers: this.instance.config.cdn.headers
                });
                const payload = yield http.get(this.dbPath);
                const eTag = payload.headers && payload.headers.ETag ? payload.headers.ETag : null;
                const versionId = payload.headers && payload.headers.VersionId
                    ? payload.headers.VersionId
                    : null;
                this.cache.set(payload.data, eTag, versionId);
                return payload.data;
            }
            catch (err) {
                throw err;
            }
        });
    }
    getFromOrigin() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const payload = yield this.instance.adapter.get(this.dbPath);
                this.cache.set(payload.Body, payload.ETag, payload.VersionId);
                return payload.Body;
            }
            catch (err) {
                throw err;
            }
        });
    }
    _privateMethods() {
        return {
            getFromEdge: this.getFromEdge.bind(this),
            getFromOrigin: this.getFromOrigin.bind(this)
        };
    }
}
//# sourceMappingURL=core.js.map